""":term:`BGP` session product block."""

import strawberry
from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
from pydantic import Field
from pydantic_forms.types import strEnum

from gso.utils.types.ip_address import IPAddress


@strawberry.enum
class IPFamily(strEnum):
    """Possible IP families of a :term:`BGP` peering."""

    V4UNICAST = "v4unicast"
    V6UNICAST = "v6unicast"
    V4MULTICAST = "v4multicast"
    V6MULTICAST = "v6multicast"



class BGPSessionInactive(ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="BGPSession"):
    """A :term:`BGP` session that is currently inactive. See :class:`BGPSession`."""

    peer_address: IPAddress | None = None
    families: list[IPFamily] = Field(default_factory=list)
    has_custom_policies: bool | None = None
    authentication_key: str | None = None
    multipath_enabled: bool | None = None
    send_default_route: bool | None = None
    is_multi_hop: bool = False
    is_passive: bool = False
    rtbh_enabled: bool = False
    bfd_enabled: bool = False


class BGPSessionProvisioning(BGPSessionInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A :term:`BGP` session that is currently being provisioned. See :class:`BGPSession`."""

    peer_address: IPAddress
    families: list[IPFamily]
    has_custom_policies: bool
    authentication_key: str | None
    multipath_enabled: bool
    send_default_route: bool
    is_multi_hop: bool
    is_passive: bool
    rtbh_enabled: bool
    bfd_enabled: bool


class BGPSession(BGPSessionProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A :term:`BGP` session that is currently deployed in the network."""

    #: The peering address of the session.
    peer_address: IPAddress
    #: The list of IP families enabled for this session.
    families: list[IPFamily]
    #: Whether any custom policies exist for this session.
    has_custom_policies: bool
    #: The authentication key of the :term:`BGP` session.
    authentication_key: str | None
    #: Whether multi-path is enabled.
    multipath_enabled: bool
    #: Whether we send a last resort route.
    send_default_route: bool
    #: Whether this session is multi-hop or not. Defaults to no.
    is_multi_hop: bool
    #: Whether this is a passive session.
    is_passive: bool
    #: Whether Remote Triggered Blackhole is enabled
    rtbh_enabled: bool
    #: Settings for :term:`BFD`.
    bfd_enabled: bool
