"""Service Binding Port.

A service binding port is used to logically attach an edge port to a customer service using a :term:`VLAN`.
"""

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle
from pydantic import Field

from gso.products.product_blocks.bgp_session import (
    BGPSession,
    BGPSessionInactive,
    BGPSessionProvisioning,
)
from gso.products.product_blocks.edge_port import EdgePortBlock, EdgePortBlockInactive, EdgePortBlockProvisioning
from gso.utils.shared_enums import SBPType
from gso.utils.types.ip_address import IPv4AddressType, IPV4Netmask, IPv6AddressType, IPV6Netmask
from gso.utils.types.virtual_identifiers import VLAN_ID

class BFDSettingsInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="BFDSettings"
):
    """Settings for :term:`BFD`, see :class:`BFDSettings`."""

    bfd_enabled: bool = False
    bfd_interval_rx: int | None = None
    bfd_interval_tx: int | None = None
    bfd_multiplier: int | None = None


class BFDSettingsProvisioning(BFDSettingsInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """Settings for :term:`BFD`, see :class:`BFDSettings`."""

    bfd_enabled: bool
    bfd_interval_rx: int | None = None
    bfd_interval_tx: int | None = None
    bfd_multiplier: int | None


class BFDSettings(BFDSettingsProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A set of settings for :term:`BFD`."""

    #: Whether :term:`BFD` is enabled.
    bfd_enabled: bool
    #: The :term:`BFD` interval RX, if enabled.
    bfd_interval_rx: int | None
    #: The :term:`BFD` interval TX, if enabled.
    bfd_interval_tx: int | None
    #: The :term:`BFD` multiplier, if enabled.
    bfd_multiplier: int | None


class ServiceBindingPortInactive(
    ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="ServiceBindingPort"
):
    """A Service Binding Port that's currently inactive. See :class:`ServiceBindingPort`."""

    is_tagged: bool | None = None
    vlan_id: VLAN_ID | None = None
    sbp_type: SBPType | None = None
    ipv4_address: IPv4AddressType | None = None
    ipv4_mask: IPV4Netmask | None = None
    ipv6_address: IPv6AddressType | None = None
    ipv6_mask: IPV6Netmask | None = None
    custom_firewall_filters: bool | None = None
    geant_sid: str | None = None
    bgp_session_list: list[BGPSessionInactive] = Field(default_factory=list)
    edge_port: EdgePortBlockInactive | None = None
    v4_bfd_settings: BFDSettingsInactive
    v6_bfd_settings: BFDSettingsInactive


class ServiceBindingPortProvisioning(ServiceBindingPortInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A Service Binding Port that's currently being provisioned. See :class:`ServiceBindingPort`."""

    is_tagged: bool
    vlan_id: VLAN_ID | None = None
    sbp_type: SBPType
    ipv4_address: IPv4AddressType | None = None
    ipv4_mask: IPV4Netmask | None = None
    ipv6_address: IPv6AddressType | None = None
    ipv6_mask: IPV6Netmask | None = None
    custom_firewall_filters: bool
    geant_sid: str
    bgp_session_list: list[BGPSessionProvisioning]  # type: ignore[assignment]
    edge_port: EdgePortBlockProvisioning
    v4_bfd_settings: BFDSettingsProvisioning
    v6_bfd_settings: BFDSettingsProvisioning


class ServiceBindingPort(ServiceBindingPortProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A service binding port that is actively used in the network."""

    #: Whether this :term:`VLAN` is tagged or not.
    is_tagged: bool
    #: The :term:`VLAN` ID.
    vlan_id: VLAN_ID | None = None
    #: Is this service binding port layer 2 or 3?
    sbp_type: SBPType
    #: If layer 3, IPv4 resources.
    ipv4_address: IPv4AddressType | None = None
    #: IPV4 subnet mask.
    ipv4_mask: IPV4Netmask | None = None
    #: If layer 3, IPv6 resources.
    ipv6_address: IPv6AddressType | None = None
    #: IPV6 subnet mask.
    ipv6_mask: IPV6Netmask | None = None
    #: Any custom firewall filters that the partner may require.
    custom_firewall_filters: bool
    #: The GÉANT service ID of this binding port.
    geant_sid: str
    #: The :term:`BGP` sessions associated with this service binding port.
    bgp_session_list: list[BGPSession]  # type: ignore[assignment]
    #: The Edge Port on which this :term:`SBP` resides.
    edge_port: EdgePortBlock
    #: :term:`BFD` settings for IPv4
    v4_bfd_settings: BFDSettings
    #: :term:`BFD` settings for IPv6
    v6_bfd_settings: BFDSettings
